module.exports = PanoramaModule = {
    Panorama: {
        // Global variables to store configuration
        iFrameRequestUrl: null,
        panoramaContainer: null,
        iFrameElement: null,
        panoramaClickCheckBox: null,
        closeButton: null,
        keyboardMovementEnabled: null,
        mapClickEnabled: null,
        markerImage: null,
        // Variable to store whether the panorama is loaded or not
        panoramaLoaded: false,

        /**
         * @param {number} x The x coordinate, given in epsg:4326
         * @param {number} y The y coordinate, given in epsg:4326
         */
        // Opens the panorama at the location x,y
        OpenPanoramaOnLocation: function (x, y) {
            // If panorama is closed, open it
            if (PanoramaModule.Panorama.panoramaContainer.style.display == 'none') {
                // Open panorama
                PanoramaModule.Panorama.OpenPanoramaFrame();
            }
            // If panorama is not loaded yet
            if (PanoramaModule.Panorama.panoramaLoaded == false) {
                // Repeat this function until panorama is loaded
                timer = setTimeout(function () {
                    PanoramaModule.Panorama.OpenPanoramaOnLocation(x, y);
                }, 50);
            }
            // If panorama is loaded
            else {
                // If keyboard movement is enabled
                if (PanoramaModule.Panorama.keyboardMovementEnabled == true) {
                    // Send this message once
                    PanoramaModule.Panorama.keyboardMovementEnabled = false;
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
                }
                // When the frame is loaded post message to open new coordinates
                PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ coordinates: [x, y] }, '*');
            }
        },

        // Opens the panorama iframe
        OpenPanoramaFrame: function () {
            // Fill in the iframes source
            if (PanoramaModule.Panorama.iFrameElement.src != PanoramaModule.Panorama.iFrameRequestUrl) {
                PanoramaModule.Panorama.iFrameElement.src = PanoramaModule.Panorama.iFrameRequestUrl;
            }
            // Unhide it
            PanoramaModule.Panorama.panoramaContainer.style.display = '';
            // Add event listener to close button
            PanoramaModule.Panorama.closeButton.addEventListener('click', PanoramaModule.Panorama.HidePanoramaFrame);
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker != null) {
                // Enable marker
                marker.style.display = '';
            }
        },

        // Hides the panorama frame
        HidePanoramaFrame: function (event) {
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            // Get the marker
            const marker = document.getElementById('panorama-view-cone');
            // If there is a marker
            if (marker != null) {
                // Remove marker
                marker.style.display = 'none';
            }
        },

        /**
         * @param {Object} map The map object, after initialized using: new maplibregl.Map
         * @param {Object} maplibregl The maplibregl object that is imported
         */
        // Configure MapLibre map for panorama usage, call this method in map.onload
        ConfigureMapLibreMap: function (map, maplibregl) {
            // If keyboard movement is on
            if (PanoramaModule.Panorama.keyboardMovementEnabled == true) {
                // Disable map keyboard controls
                map.keyboard.disable();
            }
            // If map is clickable
            if (PanoramaModule.Panorama.mapClickEnabled != false) {
                // Open panorama on click location
                map.on('click', function (e) {
                    // If checkbox is checked
                    if (PanoramaModule.Panorama.panoramaClickCheckBox.checked == true) {
                        // Open panorama on location
                        PanoramaModule.Panorama.OpenPanoramaOnLocation(e.lngLat.lng, e.lngLat.lat);
                    }
                })
            }
            // If marker is enabled (exists)
            if (PanoramaModule.Panorama.markerImage != null) {
                // Create new marker
                const newMarker = document.createElement('div');
                newMarker.id = 'panorama-view-cone';
                const img = new Image();
                img.addEventListener("load", function () {
                    newMarker.style.width = img.naturalWidth + 'px';
                    newMarker.style.height = img.naturalHeight + 'px';
                    newMarker.style.backgroundImage = 'url(' + img.src + ')';
                });
                img.src = PanoramaModule.Panorama.markerImage;
                //newMarker.style.backgroundImage = "url(" + panorama.markerImage + ")";
                let maplibreMarker = new maplibregl.Marker(newMarker);
                // When new location or angle comes as message
                window.addEventListener('message', function (event) {
                    if (event.data.angle != undefined && event.data.coordinates != undefined) {
                        // If panorama is open
                        if (PanoramaModule.Panorama.panoramaContainer.style.display != 'none') {
                            // Get the marker
                            const marker = document.getElementById('panorama-view-cone');
                            // If there is a marker
                            if (marker != null) {
                                // Get its coordinates
                                const markerCoordinates = maplibreMarker.getLngLat();
                                // If the new coordinates are different than the markers coordinates
                                if (markerCoordinates != null && markerCoordinates.lng != parseFloat(event.data.coordinates[0]) && markerCoordinates.lat != parseFloat(event.data.coordinates[1])) {
                                    // Center map
                                    map.flyTo({ center: event.data.coordinates });
                                }
                            }
                            else {
                                maplibreMarker.setLngLat([29, 41]).addTo(map);
                            }
                            maplibreMarker.setLngLat(event.data.coordinates).setRotation(event.data.angle);
                        }
                    }
                })
            }
        },

        /**
         * @param {String} properties.token The request token
         * @param {String} properties.environment The mode that the panorama will run on, default is deploy, test and development also available
         * @param {String} properties.panoramaType Default: 2013-2018 - The type of panorama, empty string for 2013-2018, 2018 for 2018 and 2013 for 2013
         * @param {String} properties.panoramaContainerId Default: panorama-container - The container that includes the panorama and its related controls
         * @param {String} properties.panoramaIframeId Default: panorama-iframe - Id of the iframe element
         * @param {String} properties.panoramaCloseButtonId Default: panorama-close-button - Id of the button that closes the panorama container
         * @param {Boolean} properties.mapClickEnabled Default: false - The parameter that decides whether a click on the map will open the panorama, loads default checkbox with id: "panorama-checkbox"
         * @param {String} properties.panoramaCheckBoxElementId The checkbox that enables "click map open panorama".
         * @param {String} properties.markerImage Default: https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png - The image for the view cone
         * @param {Boolean} properties.keyboardControlsEnabled Default: true - Enables or disables keyboard movement inside the panorama.
        */
        // Configure panorama
        ConfigurePanorama: function (properties) {
            // Set panoramaType to empty string if null
            if (properties.panoramaType === undefined) {
                properties.panoramaType = '';
            }

            //#region environment
            // Set request url according to env
            switch (properties.environment) {
                case "development":
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://localhost:44314/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
                    break;
                case "test":
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanoramatest.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
                    break;
                default:
                    PanoramaModule.Panorama.iFrameRequestUrl = 'https://cbspanorama.ibb.gov.tr/panorama/panorama' + properties.panoramaType + '/?token=' + properties.token;
            }
            //#endregion

            //#region  panoraa container
            // Get panorama container from document
            switch (properties.panoramaContainerId) {
                case undefined:
                    PanoramaModule.Panorama.panoramaContainer = document.getElementById('panorama-container');
                    break;
                default:
                    PanoramaModule.Panorama.panoramaContainer = document.getElementById(properties.panoramaContainerId);
            }
            // Hide container
            PanoramaModule.Panorama.panoramaContainer.style.display = 'none';
            //#endregion

            //#region panorama iframe
            // Get panorama iframe
            switch (properties.panoramaIframeId) {
                case undefined:
                    PanoramaModule.Panorama.iFrameElement = document.getElementById('panorama-iframe');
                    break;
                default:
                    PanoramaModule.Panorama.iFrameElement = document.getElementById(properties.panoramaIframeId);
            }
            //#endregion

            //#region close button
            // Get close button from document
            switch (properties.panoramaCloseButtonId) {
                case undefined:
                    PanoramaModule.Panorama.closeButton = document.getElementById('panorama-close-button');
                    break;
                default:
                    PanoramaModule.Panorama.closeButton = document.getElementById(properties.panoramaCloseButtonId);
            }
            //#endregion

            //#region map click 
            switch (properties.mapClickEnabled) {
                case true:
                    PanoramaModule.Panorama.mapClickEnabled = properties.mapClickEnabled;
                    switch (properties.panoramaClickCheckBox) {
                        case undefined:
                            PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById('panorama-checkbox');
                            break;
                        default:
                            PanoramaModule.Panorama.panoramaClickCheckBox = document.getElementById(properties.panoramaCheckBoxElementId);
                    }
                    switch (properties.mode) {
                        case "development":
                            PanoramaModule.Panorama.markerImage = 'https://localhost:44314/images/panoiconBlue.png';
                            break;
                        case "test":
                            PanoramaModule.Panorama.markerImage = 'https://cbspanoramatest.ibb.gov.tr/images/panoiconBlue.png';
                            break;
                        default:
                            PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                    }
                    break;
                default:
                    PanoramaModule.Panorama.mapClickEnabled = false;
            }
            //#endregion

            //#region marker image
            // If marker image is specified
            switch (properties.markerImage) {
                case undefined:
                    switch (properties.environment) {
                        case "development":
                            PanoramaModule.Panorama.markerImage = 'https://localhost:44314/images/panoiconBlue.png';
                            break;
                        case "test":
                            PanoramaModule.Panorama.markerImage = 'https://cbspanoramatest.ibb.gov.tr/images/panoiconBlue.png';
                            break;
                        default:
                            PanoramaModule.Panorama.markerImage = 'https://cbspanorama.ibb.gov.tr/images/panoiconBlue.png';
                    }
                    break;
                default:
                    PanoramaModule.Panorama.markerImage = properties.markerImage;
            }
            switch (properties.keyboardMovementEnabled) {
                case undefined:
                    PanoramaModule.Panorama.keyboardMovementEnabled = true;
                    break;
                default:
                    PanoramaModule.Panorama.keyboardMovementEnabled = properties.keyboardControlsEnabled;
                    PanoramaModule.Panorama.iFrameElement.contentWindow.postMessage({ keyboardEnabled: true }, '*');
            }
            //#endregion

            // Catch the panorama loaded message
            window.addEventListener('message', function (event) {
                if (event.data.load === true) {
                    PanoramaModule.Panorama.panoramaLoaded = true;
                }
            });
        }
    }
}